/****************************************************************

file arith.c

Hashing package for Diophantine equations,
Version 1.0

Copyright (C) 2006 A.-S. Elsenhans and J. Jahnel

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Address of the authors

	A.-S. Elsenhans/ J. Jahnel
	Math. Institut der Universitaet
	Bunsenstrasze 3--5
	D-37073 Goettingen
        Germany

WWW     http://www.uni-math.gwdg.de/jahnel

*****************************************************************/


#include"arith.h"


/* Basic arithmetic functions */

/* GCD of two integers. Assumes that at least one of them is
   nonzero. */
long int gcd (long int x, long int y) {
 long int  t;

 if (x == 0) return (y);
 if (y == 0) return (x);

 if (x < 0) x = -x;
 if (y < 0) y = -y;
 while (x != 0) {
  t = y % x;
  y = x;
  x = t;
 }
 return (y);
}


/* Computes the inverse of x modulo modul. Returns 0 if x is not
   invertible. This function is more or less taken from
   Otto Forster's book. */
long int mod_inverse (long int x, long int modul) {
 long int  q11, q12, q22, q21, y, q, temp, t21, t22;

 if (modul < 0) modul = -modul;
 y = modul;
 x = x % modul;
 if (x < 0) x += modul;
 q11 = q22 = 1;
 q12 = q21 = 0;
 while (y != 0) {
  temp = y;
  q = x / y;
  y = x % y;
  x = temp;
  t21 = q21; t22 = q22;
  q21 = q11 - q*q21;
  q22 = q12 - q*q22;
  q11 = t21; q12 = t22;
 }
 if (x != 1)
  return 0;
 q11 = q11 % modul;
 if (q11 < 0)
  q11 += modul;
 return (q11);
}
