/****************************************************************

file settings.h -- For kub.

Hashing package for Diophantine equations,
Version 1.0

Copyright (C) 2006 A.-S. Elsenhans and J. Jahnel

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Address of the authors

	A.-S. Elsenhans/ J. Jahnel
	Math. Institut der Universitaet
	Bunsenstrasze 3--5
	D-37073 Goettingen
        Germany

WWW     http://www.uni-math.gwdg.de/jahnel

*****************************************************************/


#define A_LIMIT           100
#define B_LIMIT           100
/* This program searches for solutions of the Diophantine equations
       ax**3 = by**3 + z**3 + v**3 + w**3
   where a \in [1,A_LIMIT] and b \in [1,B_LIMIT].

   A_LIMIT and B_LIMIT must not be bigger than 31767. */

#define SUCHWEITE        5000
/* SUCHWEITE must not be bigger than 31767.

   The program searches for solutions such that x, y, z, and w are
   below this limit.
   This version requires
       MAX (A_LIMIT, B_LIMIT) * SUCHWEITE**3 < 2**63 * PAGE_PRIME.
   There would occur an overflow, otherwise. */

#define INTERVALL_LAENGE 1000
/* We want a separate counter for solutions of height in each of the
   intervals [i * INTERVALL_LAENGE, (i + 1) * INTERVALL_LAENGE). */

#define PAGE_PRIME       3347
/* Must be a prime 2 mod 3 in order to ensure that third roots are
   unique.

   Also, PAGE_PRIME must not be bigger than SUCHWEITE.

   Finally, PAGE_PRIME must be bigger than A_LIMIT as, in init, we
   want to invert 1, ..., A_LIMIT modulo PAGE_PRIME. */

/* For SUCHWEITE 5000, one has
         Number of entries in the hash table
       = 1.67 * 10**11 / PAGE_PRIME
   for most of the residue classes.

   If this number is >2**HASH_BREITE then the program will NOT work.
   It seems to work best for 0.3..0.6 * 2**HASH_BREITE. */


/* We write, for simplicity, all variables, z, v, and w, into the
   nebentabelle to the hash table. */
struct nebentabellen_eintrag {
 short int  z, v, w;
};

struct variablen_links {
 short int  z, v, w;
};

struct variablen_rechts {
 short int  a, b, x, y;
};
