/****************************************************************

file demo.c

Hashing package for Diophantine equations,
Version 1.0

Copyright (C) 2006 A.-S. Elsenhans and J. Jahnel

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Address of the authors

	A.-S. Elsenhans/ J. Jahnel
	Math. Institut der Universitaet
	Bunsenstrasze 3--5
	D-37073 Goettingen
        Germany

WWW     http://www.uni-math.gwdg.de/jahnel

*****************************************************************/


/* This is a simple example intending to show how to use the
   Hashing package. */

#include"hashing.h"

#define SUCHWEITE 5000


/* A counter for the number of solutions. */
long int  count;


/* post_proc is called from the Hashing package.
   It checks whether a coincidence leads to a solution. */
void post_proc (struct nebentabellen_eintrag nt,
                struct variablen_rechts r) {
 long int  x, y, z, w;

 x = nt.x; y = nt.y; z = r.z; w = r.w;
 if ((x*x*x + y*y*y == z*z*z + w*w*w) && (x < z)) {
 /* The condition x < z causes that
    a) all obvious solutions are suppressed,
    b) only one of the solutions (x, y, z, w) and (z, w, x, y) is
       printed. */
  printf ("%ld can be written as ", x*x*x + y*y*y);
  printf ("%ld**3 + %ld**3 or as %ld**3 + %ld**3\n", x, y, z, w);
  count++;
 }
}


/* Let x and y run in an iterated loop.
   Write (x, y, x**3 + y**3) into the hash table. */
void write_xy () {
 long int                     x, y, left_side;
 struct nebentabellen_eintrag             dat;

 for (x = 0; x <= SUCHWEITE; x++) {
  /* We ignore the case y > x because of symmetry. */
  for (y = 0; y <= x; y++) {
   /* There should be no overflow over \pm 2**63. */
   left_side = x*x*x + y*y*y;
   dat.x = x; dat.y = y;
   element_schreiben (dat, left_side);
  }
 }
}


/* Let z and w run in an iterated loop.
   Check whether z**3 + w**3 leads to a coincidence. */
void read_zw () {
 long int                z, w, right_side;
 struct variablen_rechts              dat;

 for (z = 0; z <= SUCHWEITE; z++) {
  /* We ignore w > z because of symmetry. */
  for (w = 0; w <= z; w++) {
   /* There should be no overflow over \pm 2**63. */
   right_side = z*z*z + w*w*w;
   dat.z = z; dat.w = w;
   element_lesen (dat, right_side, post_proc);
  }
 }
}


/* Calling the functions delivered by the Hashing package plus the
   "user written" functions write_xy and read_zw. */
int main () {
 count = 0;

 /* Initializes the Hashing package. */
 hashing_init ();

 /* Clears the hash table and initializes a new writing cycle. */
 schreiben_init ();
 write_xy ();
 /* Finishing the writing cycle. Clearing all buffers. */
 schreiben_beenden ();

 /* Initializes the reading cycle. */
 lesen_init ();
 read_zw ();
 /* Finishing the reading cycle. Clearing all buffers.*/
 lesen_beenden (post_proc);

 /* Finishes usage of the Hashing package for this time. */
 hashing_beenden ();

 printf ("%ld solutions found.\n", count);
 return 0;
}
