/****************************************************************

file settings.h -- For quart.

Hashing package for Diophantine equations,
Version 1.0

Copyright (C) 2006 A.-S. Elsenhans and J. Jahnel

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Address of the authors

	A.-S. Elsenhans/ J. Jahnel
	Math. Institut der Universitaet
	Bunsenstrasze 3--5
	D-37073 Goettingen
        Germany

WWW     http://www.uni-math.gwdg.de/jahnel

*****************************************************************/


#include<gmp.h>
#include"gmp_headers.h"
#include GMP_IMPL
#include LONGLONG

#define A_LIMIT                 100
#define B_LIMIT                 100
/* A_LIMIT and B_LIMIT must not be bigger than 255.

   This program searches for solutions of the Diophantine equations
       ax**4 = by**4 + z**4 + v**4 + w**4
   where a \in [1,A_LIMIT] and b \in [1,B_LIMIT]. */

#define SUCHWEITE            100000L
/* SUCHWEITE must not be bigger than 1048575.

   The program searches for solutions such that x, y, z, and w are
   below this limit.
   This version requires
        | A_LIMIT * SUCHWEITE**4 - B_LIMIT * SUCHWEITE**4 |
      < 2**64 * PAGE_PRIME .
   There would occur an overflow, otherwise. */

#define PAGE_PRIME            60083
/* Must be a prime 3 mod 4 in order to ensure that fourth roots are
   unique up to sign.

   Also, PAGE_PRIME must not be bigger than 65535.

   Finally, PAGE_PRIME must be bigger than A_LIMIT as, in init, we
   want to invert 1, ..., A_LIMIT modulo PAGE_PRIME. */

/* For SUCHWEITE 100000 and A_LIMIT = B_LIMIT = 100, one has
         Number of entries in the hash table
       = 1.907 * 10**12 / PAGE_PRIME
   for most of the residue classes.

   If this number is > 2**HASH_BREITE then the program will NOT work.
   It seems to work best for 0.3..0.6 * 2**HASH_BREITE. */

#define WANT_OUTPUT_EACH_SOLUTION 1
/* 0 or 1 (meaning false or true) depending on what you want. */


/* A nebentabellen_eintrag contains a, b, mix_xy_high which is the
   mixture of the four upper bits of x with the four upper bits
   of y, (x mod 2**16), and (y mod 2**16).
   This means, we store in a packed form the 8-bit integers a and b
   and the 20-bit integers x and y.
   We prefer a struct instead of many arrays. */
struct nebentabellen_eintrag {
 unsigned char      a, b, mix_xy_high;
 unsigned short int              x, y;
};

struct variablen_links {
 unsigned int   x, y;
 unsigned char  a, b;
};

struct variablen_rechts {
 unsigned int  z, v, w;
};

struct alle_variablen {
 struct variablen_links  l;
 struct variablen_rechts r;
};
